#
# This is the SPEC file for creating binary and source RPMs for the VMs.
#
#
# The Qubes OS Project, http://www.qubes-os.org
#
# Copyright (C) 2010  Joanna Rutkowska <joanna@invisiblethingslab.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#


# default value in case of no qubes-builder's one

Name:		qubes-gui-agent
Version:	4.2.17
Release:	1%{?dist}
Summary:	The Qubes GUI Agent for AppVMs

Group:		Qubes
Vendor:		Invisible Things Lab
License:	GPL
URL:		https://www.qubes-os.org


%if %{?!centos:9}%{?centos} >= 9
%global enable_pipewire 1
%else
%global enable_pipewire 0
%endif
%define pa_ver_full %((pkg-config --modversion libpulse 2>/dev/null || echo 0.0) | cut -d "-" -f 1)
%define pa_module_dir %(pkg-config --variable=modlibexecdir libpulse)
%if 0%{?is_opensuse}
%define xinitrc_dir /usr/etc/X11/xinit/xinitrc.d
%define limits_dir /usr/etc/security/limits.d
%else
%define xinitrc_dir /etc/X11/xinit/xinitrc.d
%define limits_dir /etc/security/limits.d
%endif

BuildRequires:	gcc
BuildRequires:	libX11-devel
BuildRequires:	libXcomposite-devel
BuildRequires:	libXdamage-devel
BuildRequires:	libXfixes-devel
BuildRequires:	libXt-devel
BuildRequires:	libtool-ltdl-devel
BuildRequires:	libtool
%if 0%{?is_opensuse}
BuildRequires:	libpulse-devel >= 0.9.21
BuildRequires:	libpulse-devel <= 16.99.1
BuildRequires:	xorg-x11-server-sdk
BuildRequires:	libgbm-devel
%else
BuildRequires:	pulseaudio-libs-devel >= 0.9.21, pulseaudio-libs-devel <= 17.0
BuildRequires:	xorg-x11-server-devel
BuildRequires:	mesa-libgbm-devel
%endif
BuildRequires:	pam-devel
BuildRequires:	qubes-libvchan-devel
BuildRequires:	qubes-gui-common-devel >= 4.1.0
BuildRequires:	qubes-db-devel
BuildRequires:	xen-devel
BuildRequires:	systemd-rpm-macros
%if 0%{?is_opensuse}
# for directory ownership
BuildRequires:  xinit
BuildRequires:	qubes-core-qrexec
BuildRequires:	qubes-core-agent
Requires(post): %fillup_prereq
%endif
Requires:	Xorg %(xserver-sdk-abi-requires ansic)
Requires:	Xorg %(xserver-sdk-abi-requires videodrv)
Requires:	qubes-core-vm >= 3.0.14
Requires:	xorg-x11-xinit
# qubesdb-read --wait option
Requires:	qubes-db >= 4.1.4
Requires:	python%{python3_pkgversion}-xcffib
Requires:   xorg-x11-server-Xorg
Requires:   xorg-x11-server-Xephyr
Requires:   setxkbmap
Requires:   xsetroot
Requires:   xrdb
Requires:   xmodmap
Requires:   xev
Requires:   xdpyinfo
Requires:   xprop
Requires:   xrandr

Provides:   qubes-gui-vm = %{version}-%{release}
Obsoletes:  qubes-gui-vm < 4.0.0

Source0: %{name}-%{version}.tar.gz

Requires: (%{name}-selinux if selinux-policy)

%package -n pulseaudio-qubes
Summary: Audio support for Qubes VM
# The vchan sink needs .h files from pulseaudio sources
# that are not exported by any *-devel packages; thus they are internal and
# possible to change across version. They are copied to gui git.
# It is possible that our code will work fine with any later pulseaudio
# version; but this needs to be verified for each pulseaudio version.
Requires:	pulseaudio = %{pa_ver_full}
# Needed so that systemd can manage PulseAudio
Conflicts:	qubes-core-agent < 4.2.5
Conflicts:  qubes-gui-vm < 4.0.0
%if 0%{?is_opensuse}
# for directory ownership
BuildRequires: pulseaudio
%endif

%description -n pulseaudio-qubes
Pulseaudio module to enable sound support in Qubes VM

%if %enable_pipewire
%package -n pipewire-qubes
Summary:       PipeWire audio support for Qubes VMs
Requires:      pipewire
BuildRequires: pipewire-devel
BuildRequires: qubes-db-devel
BuildRequires: qubes-libvchan-devel
%if 0%{?is_opensuse}
# for directory ownership
BuildRequires: pipewire
%endif

%description -n pipewire-qubes
PipeWire module to enable sound support in Qubes VMs.  Unlike the
PulseAudio support in the pulseaudio-qubes package, this works with
the stock PipeWire service and does not depend on internal PipeWire
headers.  It also promises lower latency than PulseAudio, at least
once the audio daemon supports low latency.
%endif

%package xfce
Summary: XFCE desktop support for Qubes VM

%description xfce
XFCE desktop support for Qubes VM

%package -n qubes-gui-vnc
Summary: Remote GuiVM support using VNC
Requires: x11vnc
%if 0%{?is_opensuse}
# for directory ownership
BuildRequires: systemd
%endif

%description -n qubes-gui-vnc
Remote GuiVM support using VNC

%description
The Qubes GUI agent that needs to be installed in VM in order to provide the
Qubes fancy GUI.

%prep
%setup -q

rm -f pulse/pulsecore
ln -s pulsecore-%{pa_ver_full} pulse/pulsecore

%build
%{?set_build_flags}
#make clean
make appvm

%install
rm -rf -- "$RPM_BUILD_ROOT"
make install "DESTDIR=$RPM_BUILD_ROOT" \
                     LIBDIR=%{_libdir} \
                     DATADIR=%{_datadir} \
                     SYSLIBDIR=/usr/lib \
                     XINITRCDIR=%{xinitrc_dir} \
                     LIMITSDIR=%{limits_dir} \
                     PA_VER_FULL=%{pa_ver_full} \
                     ENABLE_SELINUX=%{with selinux} \
                     UNITDIR=%{_unitdir} \
                     USERUNITDIR=%{_userunitdir} \
                     USERPRESETDIR=%{_userpresetdir} \
                     UDEVRULESDIR=%{_udevrulesdir}

%if 0%{?is_opensuse}
mkdir -p $RPM_BUILD_ROOT/usr/lib/pam.d
mv $RPM_BUILD_ROOT/etc/pam.d/* $RPM_BUILD_ROOT/usr/lib/pam.d/
mkdir -p $RPM_BUILD_ROOT%{_fillupdir}
mv $RPM_BUILD_ROOT/etc/sysconfig/desktop \
    $RPM_BUILD_ROOT%{_fillupdir}/desktop.qubes-gui-agent
%endif

%post
if [ -x /bin/systemctl ] && readlink /sbin/init | grep -q systemd; then
    /bin/systemctl enable qubes-gui-agent.service 2> /dev/null
fi

%if 0%{?is_opensuse}
%{fillup_and_insserv}
%endif

%post -n pulseaudio-qubes

sed -i '/^autospawn/d' /etc/pulse/client.conf
echo autospawn=no >> /etc/pulse/client.conf

# simplified version of preset_units() from core-agent-linux
preset_units() {
    local represet=
    cat "$1" | while read action unit_name
    do
        if [ "$action" = "#" -a "$unit_name" = "Units below this line will be re-preset on package upgrade" ]
        then
            represet=1
            continue
        fi
        echo "$action $unit_name" | grep -q '^[[:space:]]*[^#;]' || continue
        [ -n "$action" -a -n "$unit_name" ] || continue
        if [ "$2" = "initial" -o "$represet" = "1" ]
        then
            systemctl --no-reload --global preset "$unit_name" >/dev/null 2>&1 || :
        fi
    done
}

if [ $1 -eq 1 ]
then
    preset_units /usr/lib/systemd/user-preset/75-pulseaudio-qubes.preset initial
else
    preset_units /usr/lib/systemd/user-preset/75-pulseaudio-qubes.preset upgrade
fi

%postun -n pulseaudio-qubes

for unit_name in pipewire.socket pipewire.service \
                 pipewire-pulse.socket pipewire-pulse.service \
                 wireplumber.service; do
    systemctl --no-reload --global preset "$unit_name" >/dev/null 2>&1 || :
done

%if %enable_pipewire
%post -n pipewire-qubes
%systemd_user_post pipewire.service pipewire.socket wireplumber.service

%preun -n pipewire-qubes
%systemd_user_preun pipewire.service pipewire.socket wireplumber.service
%endif

%preun
if [ "$1" = 0 ] ; then
	chkconfig qubes-gui-agent off
    [ -x /bin/systemctl ] && /bin/systemctl disable qubes-gui-agent.service
    /usr/bin/glib-compile-schemas %{_datadir}/glib-2.0/schemas &> /dev/null || :
fi

%posttrans
    /usr/bin/glib-compile-schemas %{_datadir}/glib-2.0/schemas &> /dev/null || :

if [ -d '/etc/init.d' ] && ! [ "$(ls -A /etc/init.d/ 2> /dev/null)" ]; then
    rmdir /etc/init.d
fi

%triggerin -n pulseaudio-qubes -- pulseaudio-libs

sed -i '/^autospawn/d' /etc/pulse/client.conf
echo autospawn=no >> /etc/pulse/client.conf

%clean
rm -rf $RPM_BUILD_ROOT
rm -f %{name}-%{version}

%files
%defattr(-,root,root,-)
%_bindir/qubes-gui
%_bindir/qubes-gui-runuser
%_bindir/qubes-session
%_bindir/qubes-run-xorg
%_bindir/qubes-run-xephyr
%_bindir/qubes-start-xephyr
%_bindir/qubes-change-keyboard-layout
%_bindir/qubes-set-monitor-layout
%_libdir/xorg/modules/drivers/qubes_drv.so
%_libdir/xorg/modules/drivers/dummyqbs_drv.so
%_libdir/libxf86-qubes-common.so
%attr(0644,root,root) /etc/X11/xorg-qubes.conf.template
/etc/profile.d/qubes-gui.sh
/etc/profile.d/qubes-gui.csh
%if 0%{?is_opensuse}
/usr/lib/pam.d/qubes-gui-agent
%else
%config /etc/pam.d/qubes-gui-agent
%endif
%config %{limits_dir}/90-qubes-gui.conf
%config /etc/xdg/Trolltech.conf
%{xinitrc_dir}/20qt-x11-no-mitshm.sh
%{xinitrc_dir}/20qt-gnome-desktop-session-id.sh
%{xinitrc_dir}/50guivm-windows-prefix.sh
/etc/qubes-rpc/qubes.SetMonitorLayout
/etc/qubes-rpc/qubes.GuiVMSession
/etc/qubes/post-install.d/20-qubes-guivm-gui-agent.sh
%if 0%{?is_opensuse}
%{_fillupdir}/desktop.qubes-gui-agent
%else
%config /etc/sysconfig/desktop
%endif
%_unitdir/qubes-gui-agent.service
%_udevrulesdir/70-master-of-seat.rules
%_udevrulesdir/90-qubes-virtual-input-device.rules
%_sysctldir/30-qubes-gui-agent.conf
/usr/lib/qubes/gtk4-workarounds.sh
/usr/lib/qubes/qubes-gui-agent-pre.sh
/usr/lib/qubes/qubes-keymap.sh
/usr/lib/qubes/qubes-xorg-wrapper
%{_datadir}/glib-2.0/schemas/20_qubes-gui-vm.gschema.override
/usr/lib/qubes/icon-sender
/etc/xdg/autostart/qubes-gtk4-workarounds.desktop
/etc/xdg/autostart/qubes-icon-sender.desktop
/etc/xdg/autostart/qubes-qrexec-fork-server.desktop
/etc/xdg/autostart/qubes-keymap.desktop
%dir /var/log/qubes

%files -n pulseaudio-qubes
%_tmpfilesdir/qubes-pulseaudio.conf
/etc/pulse/qubes-default.pa
%_bindir/start-pulseaudio-with-vchan
%{pa_module_dir}/module-vchan-sink.so
/etc/xdg/autostart/qubes-pulseaudio.desktop
/usr/lib/systemd/user-preset/75-pulseaudio-qubes.preset
%dir /usr/lib/systemd/user/pulseaudio.service.d
/usr/lib/systemd/user/pulseaudio.service.d/30_qubes.conf

%if %enable_pipewire
%files -n pipewire-qubes
# should be owned by pipewire, but it isn't
%dir %_datadir/pipewire/pipewire.conf.d
%_datadir/pipewire/pipewire.conf.d/30_qubes.conf
%_userpresetdir/74-qubes-vm.preset
%_libdir/pipewire-0.3/libpipewire-module-qubes.so
%dir %_defaultlicensedir/pipewire-qubes
%_defaultlicensedir/pipewire-qubes/COPYING
/etc/qubes/post-install.d/20-qubes-pipewire.sh

# Works with ordinary PipeWire service :)
%endif

%files xfce
%{xinitrc_dir}/60xfce-desktop.sh

%files -n qubes-gui-vnc
#/lib/systemd/system/qubes-gui-vncserver.service
%attr(0644,root,root) /etc/X11/xorg-qubes-x11vnc.conf.template
%dir %{_unitdir}/lightdm.service.d
%{_unitdir}/lightdm.service.d/qubes-guivm-vnc.conf
%_bindir/qubes-run-x11vnc

%changelog
* Tue Jul 30 2024 Qubes OS Team <qubes-devel@googlegroups.com>
- For complete changelog see: https://github.com/QubesOS/qubes-gui-agent-linux

* Tue Jul 30 2024 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 5512d61
- version 4.2.17

* Tue Jul 30 2024 Simon Gaiser <simon@invisiblethingslab.com> - 69c42d3
- xf86-input-mfndev: Block generation of key repeat events

* Sun Jul 28 2024 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - c8e4877
- Prevent xfsettingsd re-enabling key auto-repeat feature

* Fri Jul 05 2024 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 8f5f47d
- version 4.2.16

* Mon Jun 24 2024 Ben Grande <ben.grande.b@gmail.com> - c76a730
- Use software rendering for Gio apps by default

* Mon Jun 17 2024 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 66161c3
- version 4.2.15

* Fri Jun 14 2024 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 4b71d95
- Run x11vnc with correct SELinux context

* Wed Jun 12 2024 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - b19a4ee
- Fix missing glamoregl module in Xorg config for VNC GUIVM

* Mon May 06 2024 Kamil Aronowski <kamil.aronowski@yahoo.com> - 7a9ed34
- Add xorg-xrandr dependence for vm-archlinux

* Fri Apr 26 2024 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 2d5273b
- version 4.2.14

* Fri Apr 26 2024 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - 29fc016
- Watch qdb entry being set and modified on start or while playing

* Wed Mar 06 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - a728eda
- Allow carriage return in secure clipboard

* Tue Mar 05 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - 34fcc00
- Allow Qubes group to use all realtime priorities

* Tue Feb 27 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - dfcfc12
- Clarify input vs output streams

* Tue Feb 06 2024 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 02c85ab
- rpm: fix conditional pipewire build

* Mon Feb 05 2024 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 25b2456
- version 4.2.13

* Mon Feb 05 2024 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 42b6640
- qubes-gui-runuser: fix parameters order for calloc

* Fri Feb 02 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - 92a144b
- Fix audio quality problems

* Fri Feb 02 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - 589bd4e
- Pass streams to stream callbacks

* Fri Feb 02 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - f185dd7
- Avoid crashing during shutdown

* Fri Feb 02 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - bc3fd9a
- Do not rely on pw_context_get_support() on modern PipeWire

* Fri Feb 02 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - 5e8b0cb
- Drop support for PipeWire < 0.3.30

* Fri Feb 02 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - d46c75a
- Fix package descriptions

* Fri Feb 02 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - fe5428f
- Set node.rate for streams

* Fri Feb 02 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - de30dd2
- Add utility program to run any PipeWire stream

* Fri Feb 02 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - 454c637
- Use libvchan_send and libvchan_recv for all-or-nothing ops

* Fri Feb 02 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - 59b39bc
- Allow capturing vchan size in one go

* Fri Feb 02 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - 147d4ea
- Better error reporting

* Fri Feb 02 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - 602d8da
- Avoid zeroing out recorded samples if underrun

* Fri Feb 02 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - f21c949
- Atomically send initial stream states

* Fri Feb 02 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - d91ef65
- Use a #define for the stream rate

* Fri Feb 02 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - 8632a4e
- Do not send QUBES_PA_SINK_UNCORK_CMD on overrun

* Fri Feb 02 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - 62bc13c
- Fix error message when spa_format_parse() fails

* Fri Feb 02 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - 970526f
- Downgrade unknown ID errors to warnings

* Fri Feb 02 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - 1908dfb
- Reduce usage of confusing 'direction' flag

* Fri Feb 02 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - 14e1fde
- Cleanup license metadata

* Thu Feb 01 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - d40d142
- Obtain rate_match and position SPA IOs

* Thu Feb 01 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - 0b72166
- Only use trace logs in the realtime thread

* Thu Feb 01 2024 Demi Marie Obenour <demi@invisiblethingslab.com> - 647399a
- Fix buffer size calculation error

* Wed Jan 24 2024 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - d4b8cdf
- version 4.2.12

* Mon Jan 15 2024 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 82823f6
- archlinux: drop pipewire<1:0.4 dependency, part 2

* Mon Jan 15 2024 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 7e718db
- archlinux: drop pipewire<1:0.4 dependency

* Sat Jan 13 2024 fepitre-bot <fepitre-bot@qubes-os.org> - e2049df
- Add pulseaudio-17.0 headers

* Thu Jan 11 2024 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 58d30e5
- ci: add job for running openQA directly from a PR

* Wed Dec 13 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 37fb192
- Add comments describing thread synchronization

* Tue Dec 12 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 907acbc
- Add the PW_STREAM_MAP_BUFFERS flag

* Mon Nov 20 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 12638a4
- version 4.2.11

* Sat Nov 18 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - bd627d2
- rpm: use fillup tool to manage /etc/sysconfig/desktop on openSUSE

* Sat Nov 18 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - f07a2b7
- rpm: put PAM config in /usr/lib/pam.d on openSUSE

* Sat Nov 18 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 407ebc0
- rpm: drop explicit dependency on libXfixes

